@@ -93,7 +93,7 @@ setMethod("export", c("GenomicRanges", "BEDFile"),
             } else {
               toCSV <- function(x) {
                 if (is(x, "IntegerList")) {
-                  x <- unlist(lapply(x, paste, collapse = ","), use.names=FALSE)
+                  x <- unstrsplit(CharacterList(x), ",")
                 } else if (!is.character(x) && !is.null(x))
                   stop("Could not convert block coordinates to CSV")
                 x
@@ -203,14 +203,14 @@ setMethod("export", c("UCSCData", "BEDFile"),
             invisible(con)
           })
 
-setMethod("export", c("GRangesList", "BEDFile"),
+setMethod("export", c("CompressedGRangesList", "BEDFile"),
           function(object, con, format, ...) {
             object <- asBED(object)
             callGeneric()
           })
 
-setMethod("export", c("GenomicRangesList", "BEDFile"),
-          .export_GenomicRangesList_RTLFile)
+setMethod("export", c("SimpleGRangesList", "BEDFile"),
+          .export_SimpleGRangesList_RTLFile)
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Import
@@ -243,11 +243,12 @@ setMethod("import", "BEDFile",
           function(con, format, text, trackLine = TRUE,
                    genome = NA, colnames = NULL,
                    which = NULL, seqinfo = NULL, extraCols = character(),
-                   sep = c("\t", ""))
+                   sep = c("\t", ""), na.strings=character(0L))
           {
             if (!missing(format))
               checkArgFormat(con, format)
             sep <- match.arg(sep)
+            stopifnot(is.character(na.strings), !anyNA(na.strings))
             file <- con
             m <- manager()
             con <- queryForConnection(m, con, which)
@@ -328,7 +329,8 @@ setMethod("import", "BEDFile",
               bedClasses <- ifelse(presentNames %in% colnames,
                                    presentClasses, "NULL")
               bed <- DataFrame(read.table(con, colClasses = bedClasses,
-                                          as.is = TRUE, na.strings = ".",
+                                          as.is = TRUE,
+                                          na.strings = c(".", na.strings),
                                           comment.char = "",
                                           sep = sep,
                                           quote = ""))
