@@ -11,7 +11,7 @@
 ### wraps a low-level representation of a file, currently either a
 ### path/URL or connection.
 
-setClass("RTLFile", representation(resource = "characterORconnection"),
+setClass("RTLFile", representation(resource = "character_OR_connection"),
          contains = "VIRTUAL")
 
 setClass("RTLFileList",
@@ -260,7 +260,7 @@ createResource <- function(x, dir = FALSE, content = "") {
 
 uriExists <- function(x) {
   uri <- .parseURI(x)
-  if (uriIsLocal(x)) {
+  if (uriIsLocal(uri)) {
     exists <- file.exists(uri$path)
   } else {
     txt <- getURL(x, header = TRUE)
